<html>
    <body>
        <div style="max-width: 800px">
            <h1>Django Visitor Pass</h1>
            <h2>Missing Template</h2>
            <p>
                If you are seeing this page, you are missing the "visitors/self_service_success.html"
                template. You must provide your own template.
            </p>
        </div>
    </body>
</html>
