# Generated by Django 3.1.4 on 2020-12-23 17:10

import uuid

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Visitor",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4)),
                ("first_name", models.CharField(blank=True, max_length=150)),
                ("last_name", models.CharField(blank=True, max_length=150)),
                ("email", models.EmailField(db_index=True, max_length=254)),
                (
                    "scope",
                    models.CharField(
                        help_text="Used to map request to view function", max_length=100
                    ),
                ),
                ("created_at", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "context",
                    models.JSONField(
                        blank=True,
                        help_text="Used to store arbitrary contextual data.",
                        null=True,
                    ),
                ),
                ("last_updated_at", models.DateTimeField(auto_now=True)),
            ],
            options={
                "verbose_name": "Visitor pass",
                "verbose_name_plural": "Visitor passes",
            },
        ),
    ]
