# Generated by Django 3.1.4 on 2021-01-03 12:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("visitors", "0002_visitorlog"),
    ]

    operations = [
        migrations.AddField(
            model_name="visitor",
            name="is_active",
            field=models.BooleanField(
                default=True,
                help_text="Set to False to disable the visitor link and prevent further access.",
            ),
        ),
    ]
