# Generated by Django 3.1.4 on 2021-01-03 12:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("visitors", "0003_visitor_is_active"),
    ]

    operations = [
        migrations.AddField(
            model_name="visitor",
            name="expires_at",
            field=models.DateTimeField(
                blank=True,
                help_text="After this time the link can no longer be used - defaults to VISITOR_TOKEN_EXPIRY.",
                null=True,
            ),
        ),
    ]
